/******************************************************************************
PDR 6409

The MEMORY() function has the following limitations on systems which have more than 2 Gb RAM installed:

1. The maximum value returned for any of the memory types is 4194304 Kb (4 Gb).
   This is the maximum amount of memory which can be addressed by a 32-bit process, and hence a system limitation.

2. If the computer has between 2 and 4 Gb of physical RAM installed, calling Memory(MEM_RAM_TOTAL) always returns a rounded value of 2097152 Kb (2 Gb).
   If more than 4 Gb of physical RAM is installed, Memory(MEM_RAM_TOTAL) returns the maximum memory addressable by a 32-bit process, see above.

3. If the amount of physical memory currently available is between 2 and 4 Gb, Memory(MEM_RAM_AVAIL) returns 2097152 Kb (2 Gb).

PDR 6238
RollupId : 34, 36 or 50
XppRt1.dll

*******************************************************************************/
#include "os.ch"

PROCEDURE Main
LOCAL aBig1  := {}
LOCAL nSpace := 41
LOCAL nKb    := 1024
LOCAL nMax   := 102400000
LOCAL i
LOCAL cMemo
LOCAL nStart
LOCAL bSaveError

   CLS
   SET ALTER TO XB_RAM.TXT
   SET ALTER ON

   ? OS()
   ? ""
   ? "Element String",nMax/1024000,"MB"
   ? "RAM in MB"
   ? SUBSTR("Freier virtueller Speicher"               +SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_VIRT_AVAIL)/nKb),10,0)  // 197504 - Mehr als 100MB des virtuellen Speichers ist belegt.
   ? SUBSTR("Verfgbarer virtueller Gesamtspeicher"    +SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_VIRT_TOTAL)/nKb),10,0)  // 301268 - Das Pagefile ist ca. 300MB gro.
   ? SUBSTR("Freier Hauptspeicher (RAM)"               +SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_RAM_AVAIL) /nKb),10,0)  // 63520  - Fast die Hlfte des RAM ist z.Z. unbenutzt.
   ? SUBSTR("Installierter Hauptspeicher (RAM)"        +SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_RAM_TOTAL) /nKb),10,0)  // 130484 - Der PC hat 128MB RAM installiert.
   ShowMem()

   ? "create 1000x1000 Array "+TIME()

   bSaveError := ErrorBlock()
   ErrorBlock( {|e| Break(e)} )

   BEGIN SEQUENCE
      nStart := SECONDS()
      FOR i := 1 TO 100
         // each add 20 MB
         AddBig(aBig1,nMax,i)
         SLEEP(0)
      NEXT
   RECOVER
      ErrorBlock( bSaveError )
      ? ""
      ? "Error nach "+STR(i)+" Durchlufen"
   END SEQUENCE
   ErrorBlock( bSaveError )
   ? ""
   ? TIME()+" nach "+STR(SECONDS()-nStart)
   ? SUBSTR("Lnge Array"+SPACE(nSpace),1,nSpace)+" : "+STR(LEN(aBig1))
   ? i*nMax/1024000,"MB"
   ? ""
   ShowMem()
   ? ""

   aBig1 := {}

   SET ALTER OFF
   SET ALTER TO

   WAIT

   cMemo := MEMOREAD("XB_RAM.TXT")
   FERASE("XB_RAM.TXT")
   MEMOWRIT("XB_RAM.TXT",ConvToAnsiCP(cMemo))
   RunShell("XB_RAM.TXT","Notepad.exe",.T.)

RETURN

PROCEDURE AddBig(aBig1,nMax,nCount)
LOCAL i,j
LOCAL nStart
LOCAL nSpace := 41
LOCAL nChr := RandomInt( 65,90 )
LOCAL cText  := REPLICATE(CHR(nCHR),nMax) // 100MB String

   ? ""
   ? "Count "+STR(nCount)
   nStart := SECONDS()
   FOR i := 1 TO 1000
      FOR j := 1 TO 1000
         AADD(aBig1,cText)
         SLEEP(0)
      NEXT
      SLEEP(0)
   NEXT
   ? TIME()+" nach "+STR(SECONDS()-nStart)
   ? SUBSTR("Lnge Array"+SPACE(nSpace),1,nSpace)+" : "+STR(LEN(aBig1))
   ShowMem()

RETURN

FUNCTION ShowMem()
LOCAL nSpace := 41
LOCAL nKb    := 1024
LOCAL nUsed  := Memory(MEM_PROC_TOTAL)-Memory(MEM_PROC_AVAIL)/nKb

   ? SUBSTR("Fr den Proze verfgbarer Gesamtspeicher"+SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_PROC_TOTAL)/nKb),10,0)  // 58228  - 58MB Systemspeicher sind fr Proze verfgbar
   ? SUBSTR("Freier Speicher fr den aktuellen Proze" +SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_PROC_AVAIL)/nKb),10,0)
   ? SUBSTR("Xbase++ App nutzt"                        +SPACE(nSpace),1,nSpace)+" : "+STR((Memory(MEM_PROC_TOTAL)-Memory(MEM_PROC_AVAIL))/nKb,10,0)  // 24102  - 24MB Systemspeicher durch Proze angefordert

RETURN nUsed

*
*
*